#include <stdio.h>
#include <stdint.h>
#include <windows.h>
#include <math.h>

#include "draw.h"

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
DWORD WINAPI ProcRefresh(LPVOID);
extern DWORD WINAPI ProcMain(LPVOID);

char szClassName[] = "JulBonus";

int winx = SIZE_X;
int winy = SIZE_Y;
uint32_t *frame_buffer = NULL;
HWND hWnd;
HANDLE hThread1;
HANDLE hThread2;

int fKillThread = 0;

int win_graph(void)
{
	MSG lpMsg;
	WNDCLASS wc;
	RECT Rect;
	HANDLE hInstance;
	DWORD dwWindowStyle = WS_OVERLAPPEDWINDOW & ~(WS_THICKFRAME);
	int iWindowSizeX, iWindowSizeY;
	
	DWORD dwThreadId;
	
	frame_buffer = (uint32_t*)malloc(sizeof(DWORD) * SIZE_X * SIZE_Y);
	
	hInstance = GetModuleHandle(NULL);

	hThread1 = CreateThread( 
			NULL,                   // default security attributes
			0,                      // use default stack size  
			ProcRefresh,            // thread function name
			NULL,                   // argument to thread function 
			0,                      // use default creation flags 
			&dwThreadId);           // returns the thread identifier 
	if (hThread1 == NULL)
	{
		ExitProcess(3);
	}
	
	hThread2 = CreateThread( 
			NULL,                   // default security attributes
			0,                      // use default stack size  
			ProcMain,               // thread function name
			NULL,                   // argument to thread function 
			0,                      // use default creation flags 
			&dwThreadId);           // returns the thread identifier 
	if (hThread2 == NULL)
	{
		ExitProcess(3);
	}
	
	//    
	wc.style         = (CS_HREDRAW | CS_VREDRAW);
	wc.lpfnWndProc   = WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = NULL;
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName  = NULL;
	wc.lpszClassName = szClassName;

	Rect.left = 100;
	Rect.right = Rect.left + SIZE_X;
	Rect.top = 100;
	Rect.bottom = Rect.top + SIZE_Y;
	AdjustWindowRect(&Rect, dwWindowStyle, FALSE);
	iWindowSizeX = Rect.right - Rect.left;
	iWindowSizeY = Rect.bottom - Rect.top;
	
	//   
	if (!RegisterClass(&wc))
	{
		//MessageBox(NULL, "    !", "", MB_OK);
		return 1;
	}

	//    
	hWnd = CreateWindow( 
			szClassName,                //  
			"LED 8x8 Graph Demo",       //   
			dwWindowStyle,              //  
			50,   50,                   //    
			iWindowSizeX,  iWindowSizeY,//    
			(HWND) NULL,                //     NULL
			(HMENU) NULL,               //    
			(HINSTANCE)hInstance,       //    
			NULL);                      //    lParam   WM_CREATE
	if (!hWnd) 
	{
		MessageBox(NULL, "    !", "", MB_OK);
		return 2;
	}

	//   
	ShowWindow(hWnd, SW_SHOW); 
	UpdateWindow(hWnd);

	//       
	while (GetMessage(&lpMsg, NULL, 0, 0))
	{
		TranslateMessage(&lpMsg);
		DispatchMessage(&lpMsg);
	}
	
	free(frame_buffer);
	
	return 0;
}

int CopyFrameBuffer(HDC hdc, RECT *r)
{
	HBITMAP hbitmap;
	HDC hdct;
	BITMAPINFO bmi;
	VOID *pvbits;

	int h = r->bottom - r->top;
	int w = r->right - r->left;

	ZeroMemory(&bmi,sizeof(BITMAPINFO));
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biHeight = winy;
	bmi.bmiHeader.biWidth  = winx;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 32;

	hdct = CreateCompatibleDC(NULL);
	hbitmap = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, &pvbits, NULL,0x0);
	SelectObject(hdct, hbitmap);

	memcpy(pvbits, frame_buffer, h*w*4);

	BitBlt(hdc, r->left, r->top, r->right - r->left, r->bottom - r->top, hdct, 0, 0, SRCCOPY);

	DeleteObject(hbitmap);
	DeleteDC(hdct);

	return 1;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam)
{
	RECT Rect;
	HDC hdc;
	
	switch (messg)
	{
	case WM_CREATE:
		GetClientRect(hWnd, &Rect);
		winx = Rect.right - Rect.left;
		winy = Rect.bottom - Rect.top;
		break;
		
	case WM_PAINT:
		hdc = GetDC(hWnd);
		GetClientRect(hWnd, &Rect);
		CopyFrameBuffer(hdc, &Rect);
		ValidateRect(hWnd, &Rect);
		break;

	case WM_COMMAND: 
		switch (LOWORD(wParam)) 
		{
		default:
			break;
		} 
		break;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:
			break;
		case VK_RIGHT:
			break;
		case VK_UP:
			break;
		case VK_DOWN:
			break;
		case VK_HOME:
			break;
		case VK_END:
			break;
		case VK_INSERT:
			break;
		case VK_DELETE:
			break;
		case VK_F2:
			break;
		case VK_ESCAPE:
			goto win_exit;
		default:
			break;
		}
		break;
	
	case WM_DESTROY:
win_exit:
		fKillThread = 1;
		WaitForSingleObject(hThread1, 5000); // max 5 sec
		CloseHandle(hThread1);
		//WaitForSingleObject(hThread2, 5000); // max 5 sec
		CloseHandle(hThread2);
		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, messg, wParam, lParam);
	}
	return 0;
}

extern uint8_t abMatrix[8];

uint8_t dcycle;

DWORD WINAPI ProcRefresh( LPVOID lpParam )
{
	int i, j;
	int k = 0;
	
	Sleep(100);
	
	while (1)
	{
		//   :
		for (i = 0; i < winx*winy; i++)
		{
			frame_buffer[i] = 0x00CCCCCC;
		}
		
		//  ,  .
		//     abMatrix.
		for (i=0; i<8; i++)
			for(j=0; j<8; j++)
			{
				_putcircle(15+j*30,15+i*30,12,0,0x00FFFFFF);
				if (abMatrix[i] & (1<<(7-j)))
				{
					//_putcircle(15+j*30,15+i*30,11,1,0x0000FF00);
					_putcircle(15+j*30,15+i*30,11,1,dcycle<<8);
					
				}
				else
					_putcircle(15+j*30,15+i*30,11,1,0x00F0F0F0);
			}
		
		//    :
		InvalidateRect(hWnd, 0, TRUE);
		UpdateWindow(hWnd);
		
		Sleep(30);
		
		if (fKillThread)
			break;
		
		k++;
	}
	return 0; 
}
