/**
	@file
	main: LED Matrix 8x8 demo
	
	Pins Description:
	C8 - PD3; C7 - PD2; R2 - PD1; C1 - PD0; R4 - PC3; C6 - PC2; C4 - PC1; R1 - PC0
	R5 - PB6; R7 - PB7; C2 - PD5; C3 - PD6; R8 - PD7; C5 - PB0; R6 - PB1; R3 - PB2
	
	  ():
	(2.5 - 0.8) / 100  = 0.017 * 8 = 132
	   16.
	   : ~30.
	
	igorkov / end of 2013
 */
 
#include <stdint.h>
#if defined( WIN32 )
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#define PROGMEM
#define PGM_P uint8_t*
#define memcpy_P memcpy
#define pgm_read_word(a) (*(uint16_t*)(a))
#define pgm_read_byte(a) (*(uint8_t*)(a))
extern int win_graph(void);
#else
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/io.h>
#include <avr/wdt.h>
//#include <util/delay_basic.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#endif

#if defined( WIN32 )
void _delay_ms(uint16_t __ms)
{
	while (__ms >= 300)
	{
		__ms -= 300;
		Sleep(300);
	}
	if (__ms)
		Sleep(__ms);
}
#else
#define _delay_ms _delay_ms_int
//    FLOAT POINT,
//  !!!
void _delay_ms_int(uint16_t __ms)
{
	uint16_t __ticks;
	uint32_t __tmp = ((uint32_t)F_CPU / 4000) * __ms;
	if (__tmp > 65535)
	{
		//	__ticks = requested delay in 1/10 ms
		__ticks = (uint16_t) (__ms * 10);
		while(__ticks)
		{
			// wait 1/10 ms
			_delay_loop_2((F_CPU / 4000) / 10);
			__ticks --;
		}
		return;
	}
	else
		__ticks = (uint16_t)__tmp;
	_delay_loop_2(__ticks);
}
#endif


#include "sym.h"

uint8_t abMatrix[8];

#if defined( WIN32 )
DWORD WINAPI ProcMain( LPVOID lpParam );
#else
void ProcMain(void);
#endif

void power_off(void)
{
#if !defined( WIN32 )
	PORTD &= ~0x10; // PD4 = 0
#else
	exit(1);
#endif
}

#if !defined( WIN32 )

int pwr_cnt  = 916; // ~30  

ISR(TIMER0_OVF_vect)
{
//      .

#if 0
	pwr_cnt--;
	if (pwr_cnt == 0)
		power_off();
#endif
}

unsigned char isr_line = 0;
void dot88_clear(void);
void dot88_line(uint8_t line, uint8_t value);

ISR(TIMER2_OVF_vect)
{
	//dot88_clear();
	dot88_line(isr_line, abMatrix[isr_line]);
	
	isr_line++;
	if (isr_line == 8)
		isr_line = 0;
}

ISR(TIMER2_COMP_vect)
{
	dot88_clear();
}
#endif

/*
	DDRB   = 0xC7;
	PORTB |= 0xC7;
	
	C5 - PB0
	R6 - PB1
	R3 - PB2
	R5 - PB6
	R7 - PB7
	
	DDRC   = 0x0F;
	PORTC |= 0x0F;

	R1 - PC0
	C4 - PC1
	C6 - PC2
	R4 - PC3
	
	DDRD   = 0xEF;
	PORTD |= 0xEF;

	C1 - PD0
	R2 - PD1
	C7 - PD2
	C8 - PD3
	C2 - PD5
	C3 - PD6
	R8 - PD7
 */
void dot88_init(void)
{
#if !defined( WIN32 )
	DDRB   = 0xC7;
	DDRC   = 0x0F;
	DDRD   = 0xEF;
#endif
}

void dot88_clear(void)
{
#if !defined( WIN32 )
	// R=0
	// C=1
	PORTB = (PORTB & ~0xC6) | 0x01;
	PORTC = (PORTC & ~0x09) | 0x06;
	PORTD = (PORTD & ~0x82) | 0x6D;
#endif
}

#if !defined( WIN32 )
/*
	C1 - PD0
	C2 - PD5
	C3 - PD6
	C4 - PC1
	C5 - PB0
	C6 - PC2
	C7 - PD2
	C8 - PD3

	R1 - PC0
	R2 - PD1
	R3 - PB2
	R4 - PC3
	R5 - PB6
	R6 - PB1
	R7 - PB7
	R8 - PD7
 */
void dot88_line(uint8_t line, uint8_t value)
{
	//  :
	switch (line)
	{
	case 0:
		PORTD &= ~0x01; // C1 - PD0
		break;
	case 1:
		PORTD &= ~0x20; // C2 - PD5
		break;
	case 2:
		PORTD &= ~0x40; // C3 - PD6
		break;
	case 3:
		PORTC &= ~0x02; // C4 - PC1
		break;
	case 4:
		PORTB &= ~0x01; // C5 - PB0
		break;
	case 5:
		PORTC &= ~0x04; // C6 - PC2
		break;
	case 6:
		PORTD &= ~0x04; // C7 - PD2
		break;
	case 7:
		PORTD &= ~0x08; // C8 - PD3
		break;
	}

	//  :
	//value = ~value; // 
	if (value & 0x01) PORTC |= 0x01; // R1 - PC0
	if (value & 0x02) PORTD |= 0x02; // R2 - PD1
	if (value & 0x04) PORTB |= 0x04; // R3 - PB2
	if (value & 0x08) PORTC |= 0x08; // R4 - PC3
	if (value & 0x10) PORTB |= 0x40; // R5 - PB6
	if (value & 0x20) PORTB |= 0x02; // R6 - PB1
	if (value & 0x40) PORTB |= 0x80; // R7 - PB7
	if (value & 0x80) PORTD |= 0x80; // R8 - PD7
}
#endif

extern uint8_t symbol_table[] PROGMEM;

void set_ch(uint8_t ch)
{
	memcpy_P(abMatrix, &symbol_table[ch*8], 8);
	//pgm_read_byte(&frame_smile[1]);
	//strcpy_P(buffer, (PGM_P)pgm_read_word(&(string_table[i])));
}


void smile(void)
{
	set_ch(SYM_SMILE);
}

void duty_cycle(uint8_t value)
{
#if defined( WIN32 )
	extern uint8_t dcycle;
	dcycle = value;
#else
	OCR2 = value;
	//dot88_clear();
#endif
}

void shift_left(void)
{
	uint8_t i;
	for (i=0; i<8; i++)
	{
		abMatrix[i] <<= 1;
	}
}

void shift_right(void)
{
	uint8_t i;
	for (i=0; i<8; i++)
	{
		abMatrix[i] >>= 1;
	}
}

void shift_up(void)
{
	uint8_t i;
	for (i=0; i<7; i++)
	{
		abMatrix[i] = abMatrix[i+1];
	}
	abMatrix[i] = 0;
}

void shift_down(void)
{
	uint8_t i;
	for (i=7; i>0; i--)
	{
		abMatrix[i] = abMatrix[i-1];
	}
	abMatrix[i] = 0;
}

uint16_t timing;

// 1.    
void line(uint8_t *str, int len, char cf)
{
	uint16_t i;
	
	for (i=0; i < (uint16_t)(len-1)*8+1; i++)
	{
		uint8_t j;
		uint8_t offset = ((i-1)&0x07);
		uint8_t next = str[(i-1)/8 + 1];
		
		if (i == 0)
		{
			if (cf == 0)
			{
				for (j=0; j<8; j++)
				{
#if defined( WIN32 )
					abMatrix[j]  = symbol_table[str[0]*8+j];
#else
					abMatrix[j]  = pgm_read_byte(&(symbol_table[str[0]*8+j]));
#endif
				}
			}
		}
		else
		{
			shift_left();
			for (j=0; j<8; j++)
			{
#if defined( WIN32 )
				abMatrix[j] |= symbol_table[next*8+j] >> (7-offset);
#else
				abMatrix[j] |= pgm_read_byte(&(symbol_table[next*8+j])) >> (7-offset);
#endif
			}
		}
		
		_delay_ms(timing);
	}
}

// 2.  
void anim(uint8_t *str, int len)
{
	uint16_t i;
	for (i=0; i < len; i++)
	{
		memcpy_P(abMatrix,  &symbol_table[str[i]*8], 8);
		_delay_ms(timing);
	}
}

#define DUTY_CYCLE 0xF0

/**
	main()
 */
int main(void)
{
#if !defined( WIN32 )
	// fix power PD4 = 1;
	DDRD  |= 0x10;
	PORTD |= 0x10;
#endif
	
	/* init io */
	dot88_init();
	dot88_clear();

#if 0
	/* smile default */
	smile();
#endif

#if !defined( WIN32 )
	/* timer0 prescale set 1 - 1, 2 - 8, 3 - 64, 4 - 256, 5 - 1024 */
	/* interrup work freq: 8Mhz / 256 / 1024 = 30.5Hz */
	TCCR0 = 5;
	/* reset timer0 value */
	TCNT0 = 0;
	
	/* timer2 prescale set 1 - 1, 2 - 8, 3 - 32, 4 - 64, 5 - 128, 6 - 256, 5 - 1024 */
	/* interrup work freq: 8Mhz / 256 / 64 = 250Hz */
	TCCR2 = 4;
	/* reset timer2 value */
	TCNT2 = 0x00;
	/* setup duty cycle = 50% */
	OCR2 = DUTY_CYCLE;

	/* Enable Timers interrupts: */
	TIMSK = (1 << TOIE0) | (1 << OCIE2) | (1 << TOIE2);
	
	/* enable global interrupts */
	sei();
#else
	duty_cycle(DUTY_CYCLE);
#endif

#if defined( WIN32 )
	win_graph();
#else
	ProcMain();
#endif
	return 0;
}

uint8_t str_no[] = { SYM_SPACE, SYM_H, SYM_E, SYM_L, SYM_L, SYM_O, SYM_EX, SYM_SMILE };

#if defined( WIN32 )
DWORD WINAPI ProcMain( LPVOID lpParam )
#else
void ProcMain(void)
#endif
{
	uint8_t i;
	uint8_t duty;

	timing = 120;

#if 1
	timing = 120;
	/* out runnig line */
	//line(str_hello, sizeof(str_hello), 0);
	//line(str_happy, sizeof(str_happy), 0);
	line(str_no, sizeof(str_no), 1);
	_delay_ms(500);
	set_ch(SYM_SMILE2);
	_delay_ms(500);
	set_ch(SYM_SMILE);
#else
	smile();
#endif

	_delay_ms(3000);

#if !defined( WIN32 )
	//    :
	for (duty=240; duty > 112; duty-=16)
	{
		duty_cycle(duty);
		_delay_ms(80);
	}
	//    :
	for (        ; duty > 1; duty-=8)
	{
		duty_cycle(duty);
		_delay_ms(80);
	}
	duty_cycle(duty);
#endif

	power_off();
	while (1);
}
